import java.awt.*;
import java.awt.geom.*;

// A little bit of geometry/linear algebra in R^2
// Mostly figuring out stuff about lines specified by two points

class Geom {
    // signed area of triangle p, q, r
    public static double signedArea(Complex p,Complex q,Complex r) 
    {
	double p1, p2;
	double q1, q2;
	double r1, r2;
	p1 = p.re();
	p2 = p.im();
	q1 = q.re();
	q2 = q.im();
	r1 = r.re();
	r2 = r.im();
	return ( (q1*r2 - q2*r1) -
		 (p1*r2 - p2*r1) +
		 (p1*q2 - p2*q1) );
    }
    // which side of the oriented line pq contains r?
    // +1, -1, or 0 (if p,q,r collinear)
    public static int side(Complex p,Complex q,Complex r)
    {
	return (int)Math.signum(signedArea(p,q,r));
    }
    // Does pq intersect rs?
    public static boolean intersects(Complex p,Complex q,Complex r,Complex s)
    {
	return Line2D.linesIntersect(p.re(),p.im(),
				     q.re(),q.im(),
				     r.re(),r.im(),
				     s.re(),s.im());
	// alternate way: test sides
    }
    // Find intersectionof pq and rs
    public static Complex intersection(Complex p,Complex q,Complex r,Complex s)
    {
	Complex pq = q.sub(p);
	Complex rs = s.sub(r);

	double x1 = p.re(); double y1 = p.im();
	double x2 = q.re(); double y2 = q.im();
	double x3 = r.re(); double y3 = r.im();
	double x4 = s.re(); double y4 = s.im();

	double denom = (x2-x1)*(y4-y3) - (x4-x3)*(y2-y1);
	double t12 = ((x4-x3)*(y1-y3) - (y4-y3)*(x1-x3)) / denom;
	// double t34 = ((x1-x1)*(y1-y3) - (y2-y1)*(x1-x3)) / denom;

	return p.add(pq.realMul(t12));
    }    
}
