import java.awt.*;
import java.awt.geom.*;

class IdealTriangle extends PoincareTriangle {
    Complex center;

    // CONSTRUCTORS

    public IdealTriangle(double theta[]) {
	super(Complex.cxExp(1,theta[0]),
	      Complex.cxExp(1,theta[1]),
	      Complex.cxExp(1,theta[2]));
	center = Poincare.fromKlein( Poincare.toKlein(V[0]).add(Poincare.toKlein(V[1])).add(Poincare.toKlein(V[2])).realDiv(3.0) );
    }

    public IdealTriangle(double theta1, double theta2, double theta3) {
	super(Complex.cxExp(1,theta1),
	      Complex.cxExp(1,theta2),
	      Complex.cxExp(1,theta3));
	center = Poincare.fromKlein( Poincare.toKlein(V[0]).add(Poincare.toKlein(V[1])).add(Poincare.toKlein(V[2])).realDiv(3.0) );
    }

    // Which edge exterior contains a given point?
    // Returns index, or -1 if the point is inside.
    public int getIndex(Complex zideal)
    {
	int i;
	for (i=2; i>=0; i--) {
	    if (E[i].getSide(zideal) == -1)
		break;
	}
	return i;
    }

    // Which side does NOT intersect a given geodesic?
    // Note the segment should be a complete geodesic, since it only
    // checks endpoints.
    public int getNonIntersector(PoincareSegment g)
    {
	int i;
	for (i=2; i>=0; i--) {
	    if (! E[i].intersects(g))
		break;
	}
	return i;
    }

    public PoincareSegment getChord(PoincareSegment g)
    {
	int index = getNonIntersector(g);
	return new PoincareSegment(E[(index+1)%3].intersection(g),
				   E[(index+2)%3].intersection(g));
    }
}
